'------------------------------------------------------------------------------
' Program Name           : Lm75Full.bas
' Program Date           : October 15,1999
' Program Written By     : M. Akers Enterprises
'                          Michael W. Akers
'                          3800 Vineyard Ave. #E
'                          Pleasanton, California 94566
'                          Voice: +1 925 484 4750
'                          Email: mwakers@home.com
' Program Purpose        : This program will demonstrate how to interface to,
'                          and communicate With The National Semiconductor LM75
'                          Digital Temperature Sensor.
' Target Processor       : Atmel 89C52
'------------------------------------------------------------------------------
' Programmer           Date        Comments
' -------------------  ----------  --------------------------------------------
' Michael Akers        10/15/99    Initial creation of program.
' -----------------------------------------------------------------------------
' Define the processor (the regfile goes here!)
   $regfile = "8052.DAT"
' Define all meta-commands that must beinserted before all other commands.
' Define all subroutines
   Declare Sub Readlm75(lm75addr As Byte)
   Declare Sub Getconfig(lm75addr As Byte)
   Declare Sub Setconfig(lm75addr As Byte)
   Declare Sub Gettos(lm75addr As Byte)
   Declare Sub Settos(lm75addr As Byte)
   Declare Sub Gethyst(lm75addr As Byte)
   Declare Sub Sethyst(lm75addr As Byte)
   Declare Sub Setaddr(lm75addr As Byte , Flagrw As Byte)
   Declare Sub Resetpointer()
   Declare Sub Val2temp(lm75tmphi As Byte , Lm75tmplo As Byte , Lm75tmpsign As Bit)
' Define all variables and constants
   Dim Lm75read As Byte                                       ' Read address base
   Dim Lm75write As Byte                                      ' Write address base
   Dim Lm75addr As Byte                                       ' Lm75 Address
   Dim Lm75high As Byte                                       ' Lm75 Temperature High Byte
   Dim Lm75low As Byte                                        ' Lm75 Temperature Low Byte
   Dim Lm75sign As Bit                                        ' Lm75 Temperature Sign Bit
   Dim Lm75config As Byte                                     ' Lm75 Configuration
   Dim Lm75toshi As Byte                                      ' Lm75 Tos high byte
   Dim Lm75toslo As Byte                                      ' Lm75 Tos low byte
   Dim Lm75tossign As Bit                                     ' Lm75 Tos sign bit
   Dim Lm75thysthi As Byte                                    ' Lm75 Thyst high byte
   Dim Lm75thystlo As Byte                                    ' Lm75 Thyst low byte
   Dim Lm75thystsign As Bit                                   ' Lm75 Thyst sign bit
   Dim Flagrw As Byte                                         ' Read/Write Flag
   Dim Lm75tmphi As Byte                                      ' Lm75 Temp hi byte
   Dim Lm75tmplo As Byte                                      ' Lm75 Temp lo byte
   Dim Lm75tmpsign As Bit                                     ' Lm75 Temp sign bit
' Define all Configurations and Pin assignments
   Config Sda = P1.0
   Config Scl = P1.1
   Config I2cdelay = 1
   Config Lcd = 40 * 4
   Config Lcdpin , Db4 = P1.4 , Db5 = P1.5 , Db6 = P1.6 , Db7 = P1.7 , E = P1.3 , Rs = P1.2
' Initialize variables as needed.
   Lm75read = &B10010001
   Lm75write = &B10010000
' Program start.
Start:



Goto Start
'Program end.
' Begin subroutine section.
Sub Readlm75(lm75addr)
   Call Setaddr Lm75addr , 0
   I2cstart                                                   ' Start the I2C process.
   I2cwbyte Lm75read                                          ' Send the LM75 address and read Info.
   I2crbyte Lm75high , 8                                      ' Get 8 bits and get ACK from LM75
   I2crbyte Lm75low , 9                                       ' Get 8 bits and send NACK to LM75
   I2cstop                                                    ' Stop the I2C system
   Call Val2temp Lm75high , Lm75low , Lm75sign                ' Convert the value (if needed)
   Lm75high = Lm75tmphi
   Lm75low = Lm75tmplo
   Lm75sign = Lm75tmpsign
   Lm75read = &B10010001                                      ' Reset Lm75read
End Sub
Sub Getconfig(lm75addr)
   Call Setaddr Lm75addr , 2
   I2cstart
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000001 , 8                                    ' Set pointer register to point at the configuration register.
   I2cstart                                                   ' Restart I2C
   I2cwbyte Lm75read                                          ' Send read address
   I2crbyte Lm75config , 9                                    ' Read the config register
   I2cstop
   Resetpointer                                               ' Reset the pointer to '00'
End Sub
Sub Setconfig(lm75addr)
   Call Setaddr Lm75addr , 1
   I2cstart                                                   ' Start I2C
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000001 , 8                                    ' Set Pointer to point to Configuration register
   I2cwbyte Lm75config                                        ' Send the Lm75config byte to write
   I2cstop
   Resetpointer                                               ' Reset the pointer to '00'
End Sub
Sub Gettos(lm75addr)
   Call Setaddr Lm75addr , 2
   I2cstart
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000011 , 8                                    ' Set Pointer to Tos
   I2cstart
   I2cwbyte Lm75read                                          ' Send read address
   I2crbyte Lm75toshi , 8                                     ' Read Tos high byte
   I2crbyte Lm75toslo , 9                                     ' Read Tos low byte
   I2cstop
   Resetpointer                                               ' Reset the pointer to '00'
End Sub
Sub Settos(lm75addr)
   Call Setaddr Lm75addr , 1
   I2cstart
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000011                                        ' Set Pointer to Tos
   I2cwbyte Lm75toshi                                         ' Send Tos high byte
   I2cwbyte Lm75toslo                                         ' Send Tos low byte
   I2cstop
   Resetpointer                                               ' Reset the pointer to '00'
End Sub
Sub Getthyst(lm75addr)
   Call Setaddr Lm75addr , 2
   I2cstart
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000010 , 8                                    ' Set pointer to Thyst
   I2cstart
   I2cwbyte Lm75read                                          ' Send read address
   I2crbyte Lm75thysthi , 8                                   ' Read Thyst high byte
   I2crbyte Lm75thystlo , 9                                   ' Read Thyst low byte
   I2cstop
   Resetpointer                                               ' Reset the pointer to '00'
End Sub
Sub Setthyst(lm75addr)
   Call Setaddr Lm75addr , 1
   I2cstart
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000010                                        ' Set pointer to Thyst
   I2cwbyte Lm75thysthi                                       ' Send Thyst high byte
   I2cwbyte Lm75thystlo                                       ' Send Thyst low byte
   I2cstop
   Resetpointer                                               ' Reset the pointer to '00'
End Sub
Sub Setaddr(lm75addr , Flagrw)
   If Lm75addr <> 0 Then
      Lm75addr = Lm75addr * 2                                 ' Shift the address 1 bit left
      Select Case Flagrw
         Case 0 : Lm75read = Lm75read + Lm75addr              ' Add the address offset to the read address
         Case 1 : Lm75write = Lm75write + Lm75addr            ' Add the address offset to the write address
         Case 2 :
            Lm75read = Lm75read + Lm75addr                    ' Add the address offset to both read and write addresses
            Lm75write = Lm75write + Lm75addr
         Case Else
      End Select
   End If
End Sub
Sub Resetpointer()
' Only call this routine from within a subroutine that has called Setaddr()!
   I2cstart
   I2cwbyte Lm75write                                         ' Send write address
   I2cwbyte &B00000000 , 9                                    ' Set pointer to '00'
   I2cstop
   Lm75read = &B10010001
   Lm75write = &B10010000
End Sub
Sub Val2temp(lm75tmphi , Lm75tmplo , Lm75tmpsign)
' This routine will convert the hi and lo bytes into a temperature value
   If Lm75tmphi > 127 Then
      Lm75tmphi = Not Lm75tmphi
      Incr Lm75tmphi
      Lm75tmpsign = 1
   End If
   Lm75tmplo = Lm75tmplo And &B10000000
   If Lm75tmplo = &B10000000 Then
      Lm75tmplo = 5
   End If
End Sub
Sub Temp2val(lm75tmphi , Lm75tmplo , Lm75tmpsign)
' This routine will convert a temperature value into the prober hi and lo byte values.
   If Lm75tmpsign = 1 Then
      Lm75tmphi = Not Lm75tmphi
      Incr Lm75tmphi
   End If
   If Lm75tmplo <> 0 Then
      Lm75tmplo = &B10000000
   End If
End Sub

' Insert include files here.
' End of subroutines and actual end of program.